<?php
/**
 * This file is part of Advanced Custom Fields: Carousel.
 *
 * Advanced Custom Fields: Carousel is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 2 of the License, or any later version.
 *
 * Advanced Custom Fields: Carousel is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Advanced Custom Fields: Carousel.  If not, see <https://www.gnu.org/licenses/>.
 */

declare( strict_types=1 );

namespace AcfCarousel\Carousel;

use WP_Block;

/**
 * Gutenberg block for ACF Carousel.
 *
 * @since 20220719
 */
class Block {
	/** @var string Path to plugin’s root file. */
	private string $plugin_file;

	/**
	 * Block constructor.
	 *
	 * @param string $plugin_file The path to plugin’s root file.
	 *
	 * @since 20220719
	 */
	public function __construct( string $plugin_file ) {
		$this->plugin_file = $plugin_file;
		add_action( 'init', [ $this, 'register' ] );
	}

	/**
	 * Registers the block to Gutenberg.
	 *
	 * @return void
	 *
	 * @since 20220719
	 */
	public function register() { // phpcs:ignore NeutronStandard.Functions.LongFunction
		$plugin_dir_path = plugin_dir_path( $this->plugin_file );
		$block_type = register_block_type( $plugin_dir_path . 'build/Carousel', [
			'render_callback' => [ $this, 'render' ],
		] );
		if ( $block_type ) {
			foreach ( $block_type->editor_script_handles as $handle ) {
				wp_set_script_translations( $handle, 'acf-carousel', $plugin_dir_path . 'languages' );
			}
		}
	}

	/**
	 * Renders the block’s content.
	 *
	 * @param array $block_attributes The block’s attributes.
	 *
	 * @return string The HTML for the block.
	 *
	 * @since 20220813
	 */
	public function render( array $block_attributes, string $content, WP_Block $block ): string {
		array_walk( $block->block_type->view_script_handles, 'wp_enqueue_script' );
		array_walk( $block->block_type->style_handles, 'wp_enqueue_style' );

		$slick_settings = [
			'variableWidth' => ! ! $block_attributes['variableWidth'],
			'adaptiveHeight' => ! ! $block_attributes['adaptiveHeight'],
			'infinite' => ! ! $block_attributes['infinite'],
			'centerMode' => ! ! $block_attributes['centerMode'],
		];
		if ( ! $slick_settings['variableWidth'] ) {
			$desktop_slides = absint( $block_attributes['desktopSlides'] );
			$tablet_slides = absint( $block_attributes['tabletSlides'] );
			$mobile_slides = absint( $block_attributes['mobileSlides'] );

			$slick_settings['responsive'] = [
				[
					'breakpoint' => 1366,
					'settings' => [
						'slidesToShow' => $desktop_slides,
						'slidesToScroll' => $desktop_slides,
					],
				],
				[
					'breakpoint' => 1024,
					'settings' => [
						'slidesToShow' => $tablet_slides,
						'slidesToScroll' => $tablet_slides,
					],
				],
				[
					'breakpoint' => 600,
					'settings ' => [
						'slidesToShow' => $mobile_slides,
						'slidesToScroll' => $mobile_slides,
					],
				],
			];
		}

		return $this->render_carousel(
			$block_attributes['fieldName'],
			$block_attributes['useLightbox'] ?? false,
			$block_attributes['lightboxGallery'] ?? false,
			$slick_settings
		);
	}

	/**
	 * Renders the whole carousel.
	 *
	 * @param string $field The name of the gallery field to render as a carousel.
	 * @param bool $use_lightbox Whether to show the image in a lightbox or not.
	 * @param bool $lightbox_gallery Whether to show all images grouped in a gallery.
	 * @param array $slick_settings Extra settings for Slick library.
	 *
	 * @return string The HTML for the whole carousel, including arrows and data attributes.
	 *
	 * @since 20220813
	 */
	private function render_carousel( string $field, bool $use_lightbox, bool $lightbox_gallery, array $slick_settings = [] ): string {
		if ( ! function_exists( 'get_field' ) ) {
			return '';
		}

		$wrapper_attributes = get_block_wrapper_attributes( [
			'class' => 'acf-carousel',
		] );
		$next_arrow = $this->get_arrow( 'arrow-right', __( 'Next', 'acf-carousel' ) );
		$prev_arrow = $this->get_arrow( 'arrow-left', __( 'Previous', 'acf-carousel' ) );

		$link_attributes = '';
		if ( $use_lightbox ) {
			if ( $lightbox_gallery ) {
				$gallery = 'gallery-' . esc_attr( uniqid() );
				$link_attributes .= sprintf( ' rel="%1$s"', $gallery );
				$wrapper_attributes .= sprintf( ' data-lightbox="%1$s"', $gallery );
			} else {
				$link_attributes .= ' rel="lightbox"';
			}
		}

		ob_start();
		?>
		<div <?php echo $wrapper_attributes; ?> >
			<a class="slick-prev"><?php echo $prev_arrow; ?></a>
			<?php $slides = get_field( $field, get_the_ID() ); ?>
			<?php if ( $slides ) : ?>
				<div class="acf-carousel-slides"
					data-slick="<?php echo esc_attr( wp_json_encode( $slick_settings ) ); ?>"
				>
					<?php foreach ( $slides as $slide ) : ?>
						<div>
							<a
								href="<?php echo esc_url( $slide['url'] ); ?>"
								title="<?php esc_attr( $slide['alt'] ); ?>"
								<?php echo $link_attributes; ?>
							>
								<img src="<?php echo esc_url( $slide['sizes']['large'] ); ?>"
									alt="<?php echo esc_attr( $slide['alt'] ); ?>"/>
							</a>
						</div>
					<?php endforeach; ?>
				</div>
			<?php endif; ?>
			<a class="slick-next"><?php echo $next_arrow; ?></a>
		</div>
		<?php
		return ob_get_clean();
	}

	/**
	 * Gets the SVG image of an array icon.
	 *
	 * @param string $name The name of the arrow.
	 * @param string $alt Alternative text for the image.
	 *
	 * @return string The SVG code for the arrow image.
	 *
	 * @since 20220813
	 */
	private function get_arrow( string $name, string $alt ): string {
		return str_replace( ':alt', $alt, file_get_contents( plugin_dir_path( $this->plugin_file ) . "src/$name.svg" ) );
	}
}
